****************************************************************************
* EVKIT2.ASM
* Include file for use with Maxim 68HC16 Module Monitor rev 2.0
* Mark Underwood  4/21/1996
* (C) 1996 Maxim Integrated Products
*
*   Boot ROM     = 00000 - 0FFFF
*   User RAM     = 10000 - 1FFFF
*   Internal RAM = 20000 - 203FF
*       Stack = 20000 - 2036C
*       Reserved = 2036E - 203FF
*
*   40-pin chip selects = 7E000-7FFFF
*     pin 11 = 7E000 - 7E7FF
*     pin 12 = 7E800 - 7EFFF
*     pin 13 = 7F000 - 7F7FF
*     pin 14 = 7F800 - 7FFFF
*
*   System clock is 16.78 MHz
*   Software watchdog is enabled
*   Serial communication port is 1200 baud, 8 bits, no parity
****************************************************************************

CR      EQU     $0D
LF      EQU     $0A
CRLF    EQU     $0D0A
ESC     EQU     $1B
NULL    EQU     $00

USERRAM         EQU     $10000
USERRAMBANK     EQU     PAGE(USERRAM)
SRAM            EQU     $20000
SRAMBANK        EQU     PAGE(SRAM)

STACKBANK       EQU     SRAMBANK

CS_AREA_0       EQU     $7E000
CS_AREA_1       EQU     $7E800
CS_AREA_2       EQU     $7F000
CS_AREA_3       EQU     $7F800

* Vector table
*
* The EVKIT software that is loaded into RAM
* may use the monitor rom's services by
* calling these subroutines.
*
* The ONLY absolute addresses that are
* guaranteed are the 32 vectors listed here.
*
COLD_START      EQU     $0200   ; full reset -- destructive RAM test
WARM_START      EQU     $0204   ; warm restart -- RAM is preserved
TX_BYTE         EQU     $0208   ; transmit one byte to serial port
RX_BYTE         EQU     $020C   ; receive one byte from serial port
TX_HEX          EQU     $0210   ; transmit one byte as two characters
RX_HEX          EQU     $0214   ; recevie two characters --> one byte
TX_MESSAGE      EQU     $0218   ; transmit a null-terminated string
TX_CR           EQU     $021C   ; transmit CR and wait for an echo

RX_STRING       EQU     $0220   ; receive a string from the user
ATOI            EQU     $0224   ; interpret a string as an unsigned integer
FORMAT_UNSIGNED_DECIMAL EQU $0228 ; produce a string from an integer value
TX_DECIMAL      EQU     $022C   ; transmits an unsigned decimal 00000-65535
RX_DECIMAL      EQU     $0230   ; receives an unsigned decimal 0-65535
TX_HEX_WORD     EQU     $0234   ; transmits 4-character hex word (2 bytes)
ROMCollectCmd   EQU     $0238   ; command that collects and uploads samples
GET_ROM_REV     EQU     $023C   ; return monitor rom version number

RX_BYTE_AVAILABLE EQU   $0240   ; carry is set if a character is received
*SARclear        EQU     $0244   ; clear SAR pointed to by IY
*SARupdate       EQU     $0248   ; update IY's SAR with value in D
*SARdisplay      EQU     $024C   ; print IY's SAR to the console
*SARgetmean      EQU     $0250   ; get IY's SAR's mean value into D
*SARgetssd       EQU     $0254   ; calculate IY's SAR ssd field
SetSampleArrayStart  EQU $0258  ; ZK:IZ --> sample array start address
HandleDefaultCommand EQU $025C  ; Handle default EVKIT commands

STANDARD_KIT    EQU     $0260   ; Standard EVKIT control program
TOUPPER_A       EQU     $0264   ; convert register A to upper case
EXECUTE_DELAY_A EQU     $0268   ; delay between samples
EXECUTE_DELAY_B EQU     $026C   ; spare delay B
EXECUTE_DELAY_C EQU     $0270   ; spare delay C
EXECUTE_DELAY_D EQU     $0274   ; spare delay D
RESERVED_1E     EQU     $0278   ; reserved
RESERVED_1F     EQU     $027C   ; reserved

*  INCLUDE 'SARDEF.ASM'  ; definition of statistical accumulator record

***************************************
*
* TX_BYTE:
*
* Transmit the contents of register A
* as a single byte.  
*
* On Entry:
*   A is ASCII character to send
*
* On Exit:
*   One character has been sent
*   Software watchdog is reset
*
* If the character transmitted was CR,
* then send some null characters after it.
* The number of null characters is specified
* by variable NULLS.  If NULLS is zero, then
* no null characters are sent after CR.
*

***************************************
*
* RX_BYTE:
*
* Wait until a character is received.
*
* On Exit:
*   One character has been received
*   A contains the received character
*   B contains the received character
*   LAST_CHAR_RXD contains the received character
*   Software watchdog is reset
*
***************************************

****************************************
*
* TOUPPER_A:
*
* Convert the MSB-LOW ASCII character 
* in register A to upper case.  
*
* Lower case letters a-z are converted 
* to upper case letters A-Z.
*
* Punctuation and figures are 
* passed unchanged, with the following
* exceptions:
*
* $60  (')  is converted to $40 (@)
* $7B  ({)  is converted to $5B ([)
* $7C  ({)  is converted to $5C (\)
* $7D  (|)  is converted to $5D (])
* $7E  (~)  is converted to $5E (^)
* $7F (del) is converted to $5F (_)
*
* On Entry:
*
*   Register A = character to convert
*
* On Exit:
*
*   Register A = resulting character
*
*

***************************************
*
* TX_HEX:
*
* Transmit the contents of register A
* as two ASCII characters representing
* the hexadecimal value.
*
* On Entry:
*   A contains the value to transmit
*
* On Exit:
*   Two characters have been transmitted
*

***************************************
*
* RX_HEX:
*
* Receive two characters and interpret
* as a hexadecimal value
*
* On Exit:
*   Two characters have been received
*   A contains the received byte
*   B contains the received byte
*   LAST_HEX_RXD contains the received byte
*

***************************************
*
* TX_MESSAGE:
*
* Transmit a null-terminated string
* to the serial port.
*
* On Entry:
*   XK:IX points to a null-terminated string
*
* On Exit:
*   The string has been transmitted
*   XK:IX points past the null
*

***************************************
*
* TX_CR:
*
* Transmit a carriage return character 
* (0Dh) and wait for the host to 
* echo back a carriage return.
*

***************************************
*
* GET_ROM_REV:
*
* Puts the BOOT ROM version
* number in the D accumulator.
*
* 0000 = version 0.0
* 0001 = version 0.1
* 0100 = version 1.0
*

***************************************
*
* RX_BYTE_AVAILABLE:
*
* Determine whether a character has been received.
*
* On Exit:
*   If the carry flag is set, 
*       then a character has been received.
*       Call RX_BYTE to read the character.
*   If the carry flag is clear,
*       then no character has been received yet.
*
*   In either case, the software watchdog is reset.
*

*******************************
*
* FORMAT_UNSIGNED_DECIMAL:
*
* Format a 16-bit number into
* unsigned decimal format.
*
* On Entry:
*
*    D = value to format
*    ZK:IZ points to a 5-byte output area
*
* On Exit:
*
*    The 5-byte area specified above
*    contains the formatted value
*    as 5 ASCII characters.
*    00000
*    00001
*    ...
*    65535
*

*******************************
*
* ATOI:
*
* Convert string to integer
*
* On Entry:
*
*  ZK:IZ points to a buffer.
*
* On Success:
*
*  Carry flag is clear.
*  D contains the value of the
*  decimal number contained
*  in the buffer.
*
* On Failure:
*
*  Carry flag is set.
*   D = 0 indicates that the
*         string is not a number.
*   D = 1 indicates that the
*         number is bigger than
*         65535.
*

*******************************
*
* RX_STRING:
*
* Receive a string.
*
* On Entry:
*
*  ZK:IZ points to a buffer
*  E contains the buffer size
*
* On Exit:
*
*  The buffer contains a null-
*  terminated string obtained
*  from the user.
*
*  Input is accepted from the   
*  user until the user presses
*  the ENTER key or until the
*  buffer is full, whichever
*  happens first.
*

*******************************************
*
* TX_HEX_WORD:
*
* Print the 16-bit hexadecimal value in D.
*
* On Entry:
*
*   D = value to print (A=high byte)
*
* On Exit:
*
*   Value has been printed to serial port
*

******************************************
*
* TX_DECIMAL:
*
* Transmit the contents of register D
* as an unsigned decimal value between
* 00000 and 65535.
*
* On Entry:
*
*   D = value to print
*
* On Exit:
*
*   5-character value has been printed 
*   to serial port
*
* Uses 6-byte StrBuf as a temporary.
*
	
******************************************
*
* RX_DECIMAL:
*
* Accept up to 5 characters from the user
* as an unsigned decimal value between
* 00000 and 65535.
*
* On Entry:
*
*   no entry conditions
*
* On Success:
*
*   Carry flag is clear
*   D = value read from user
*
* On Failure:
*
*   Carry flag is set
*   D = error code returned by ATOI
*
* Uses 6-byte StrBuf as a temporary.
*

******************************************
*
* SetSampleArrayStart:
*
* Set up the start address of the sample array.
*
* Default sample array size is 8 bytes.
*
* On Entry:
*
*    ZK:IZ points to sample array starting address
* 

******************************************
*
* HandleDefaultCommand:
*
* Standard EVKIT command handler
*
* ESC+R = reset
* ESC+B = baud rate change
* Z = set the sample array size
* C = collect samples
* U = upload next sample (pointed to by ZK:IZ)
* - = back up the ZK:IZ pointer
* R = call READ ADC routine at 010008, print value in D register
* P = set, clear, float, or query I/O pins
* T = set or display timing values
*
* On Entry:
*
*   A = character command to process
*   DataArrayAddr = address of sample array start (in ZK page)
*
* On Exit:
*
*   Carry flag SET if one of the
*   default commands was handled.
*
*   Carry flag CLEAR if the command
*   was not something that we recognized.
*
*   Character in A is converted to upper case.
*

********************************************************
*
* STANDARD_KIT:
*
* Standard MAXIM Evaluation Kit code loop
*
* On Entry:
*
*   010000 = Initialization vector
*   010004 = Kit Command Handler
*   010008 = Read ADC vector
*   01000C = Kit Identification string address (in page 1)
*   01000E = Kit Help string address (in page 1)
*
* X index is used as a temporary.
* Z index is used to store sample array.
* 
* This loop does not exit.
* Control is passed back to the user code through
* the Kit Command Handler and the Read ADC subroutines.
*


**********************************************
*
* Collect an array of samples.
*
*   This command actually prompts the user
*   (via the serial port) for the number of samples
*   to collect, the radix to report the samples in,
*   and the type of triggering mechanism to use (if any).
*
*   After all of the samples have been collected,
*   they are uploaded to the host, one at a time,
*   with the pacing controlled by the host.
*
*   The trigger choices are:
*       - no trigger (collect samples as fast as possible)
*       - delay for a fixed number of cycles between readings
*       - external trigger provided on one of the GPT pins (J1 pins 27-34)
*
* On Entry:
*
*   The variable ReadRoutineAddress contains the
*     16-bit address of a subroutine in user memory
*     (10000-1FFFF) that reads a datum and returns
*     the value in the D register.
*
*   The variables DataArrayAddr and DataArrayPage
*     form the 20-bit address of the storage area
*     for the data array.
*
* On Exit:
*
*   The samples have been collected in memory
*   and have been reported back to the host.
*

********************************************
*
* Reserved area above stack
*
	ORG     $2035A

_usec:          ds 2    ; temporary used by set-delay, STANDARD_KIT
_dly_ms:        ds 2    ; temporary used by set-delay, STANDARD_KIT
_Delay_A:       ds 2    ; DELAY A inner loop, approx 4usec per iteration
_Delay_A_msec:  ds 2    ; DELAY A outer loop iteration count
_Delay_B:       ds 2    ; DELAY B
_Delay_B_msec:  ds 2    ; DELAY B
_Delay_C:       ds 2    ; DELAY C
_Delay_C_msec:  ds 2    ; DELAY C
_Delay_D:       ds 2    ; DELAY D
_Delay_D_msec:  ds 2    ; DELAY D

* Temporaries used by collect data command
*
ROMColCmdTemps:
ReadRoutineAddress:
	DS 2    ; address of ADC read routine in USER RAM (10000-1FFFF)
_ReadRoutineAddress equ $036E ; (2)
DataArrayAddr:  
	DS 2    ; 16-bit address of data array
_DataArrayAddr equ $0370 ; (2)
DataArrayPage:
	DS 1    ; address extension of data array
_DataArrayPage equ $0372 ; (1)
UploadInHex:
	DS 1    ; true if we should upload data in hex instead of in decimal
_UploadInHex equ $0373 ; (1)
DataArraySize:
	DS 2    ; number of samples in data array
_DataArraySize equ $0374 ; (2)
TriggerType:
	DS 1    ; 'D'=delay | 'E'=external trigger | 'N'=no delay
_TriggerType equ $0376 ; (1)
TriggerValue:
	DS 2    ; delay constant | external trigger mask
_TriggerValue equ $0377 ; (2)
StrBuf:
	DS 7    ; input / output string buffer used by TX_DECIMAL, RX_DECIMAL
_StrBuf equ $0379 ; (7)

	ORG     $20380
ISRSCI: DS  $08 ; interrupt service routine for serial communication interface
_ISRSCI equ $0380

ISRSPI: DS  $08 ; interrupt service routine for queued serial peripheral interface
_ISRSPI equ $0388

ISRGPT0:DS  $08 ; ISR for general purpose timer module (adjusted channel)
_ISRGPT0 equ $0390

ISRGPT1:DS  $08 ; ISR for general purpose timer module IC1
_ISRGPT1 equ $0398

ISRGPT2:DS  $08 ; ISR for general purpose timer module IC2
_ISRGPT2 equ $03A0

ISRGPT3:DS  $08 ; ISR for general purpose timer module IC3
_ISRGPT3 equ $03A8

ISRGPT4:DS  $08 ; ISR for general purpose timer module OC1
_ISRGPT4 equ $03B0

ISRGPT5:DS  $08 ; ISR for general purpose timer module OC2
_ISRGPT5 equ $03B8

ISRGPT6:DS  $08 ; ISR for general purpose timer module OC3
_ISRGPT6 equ $03C0

ISRGPT7:DS  $08 ; ISR for general purpose timer module OC4
_ISRGPT7 equ $03C8

ISRGPT8:DS  $08 ; ISR for general purpose timer module IC4/OC5
_ISRGPT8 equ $03D0

ISRGPT9:DS  $08 ; ISR for general purpose timer module TOF (timer overflow)
_ISRGPT9 equ $03D8

ISRGPTA:DS  $08 ; ISR for general purpose timer module PAOVF (overflow)
_ISRGPTA equ $03E0

ISRGPTB:DS  $08 ; ISR for general purpose timer module PAIF
_ISRGPTB equ $03E8

LAST_CHAR_RXD   DS 1 ; (1) last character received with RX_BYTE
_LAST_CHAR_RXD equ $03F0

LAST_HEX_RXD    DS 1 ; (1) last byte received with RX_HEX
_LAST_HEX_RXD equ $03F1

ExtCmdsEnabled  DS 1 ; (1) extended commands enabled
_ExtCmdsEnabled equ $03F2

*** temporary variables on the stack (13 bytes)
*
Temp1           DS 1
_Temp1 equ $03F3 ; (1)
Temp2           DS 2
_Temp2 equ $03F4 ; (2)
SRecAddr        DS 2 ; address of record
_SRecAddr equ $03F6 ; (2) address of record
CalcSCheckSum   DS 1 ; checksum calculated for incoming S-record
_CalcSCheckSum equ $03F8 ; (1) checksum calculated for incoming S-record
SRecType        DS 1 ; character '0', '1', '2', '9' to indicate record type
_SRecType equ $03F9 ; (1) character '0', '1', '2', '9' to indicate record type
SRecLength      DS 1 ; length of record
_SRecLength equ $03FA ; (1) length of record
SRecExtAddr     DS 1 ; address extension (bank) of record
_SRecExtAddr equ $03FB ; (1) ; address extension (bank) of record
NULLS           DS 1 ; number of null characters to send after CR
_NULLS equ $03FC ; (1) number of null characters to send after CR
Temp3           DS 1
_Temp3 equ $03FD ; (1)
FILL_COUNT      DS 2 ; fill memory counter
_FILL_COUNT equ $03FE ; (2) fill memory counter
*
*** do not extend this area; 203FF is the maximum address here

***************************************
*
* Fall through to main program.
*
* EVKIT program begins in user RAM.
*
*
	ORG     USERRAM        
	

