; *****************************************************************************
; *
; * hsi2c.asm (Rowley CrossWorks)
; *  
; * Maxim Integrated Products
; *   
; * DESCRIPTION
; *
; *   Contains one assembly routine:
; *      - hsi2cRecv
; *   Allows the MAXQ2000 to bit-bang a High-Speed I2C
; *   interface at 1.7MHz using a 20MHz crystal and 1.5k
; *   pull up resistors. 
; *
; * LEGAL
; *
; *   See README.txt
; *
; * REVISIONS
; *   09 08 2006 - v1.0 Initial Release (TS) See ;T
; *   10 06 2005 - v0.9 Prelim Version (PH) 
; *
; *****************************************************************************
#include "maxQ2000.inc"

; CUSTOMIZATION SECTION - START -----------------------------------------------
I2C_ADDR7  equ 0           ; The I2C addr
I2C_ADDR6  equ 1           ; 
I2C_ADDR5  equ 1           ; 
I2C_ADDR4  equ 1           ;
I2C_ADDR3  equ 1           ;
I2C_ADDR2  equ 1           ;
I2C_ADDR1  equ 1           ;

SCL        equ 0           ; SCL pin on I/O port
SDA        equ 1           ; SDA pin on I/O port

SCL_DIR MACRO x      ; SCL direction reg bit = x
   move PD6.SCL,#x         ;
ENDM                       ;

SDA_DIR MACRO x      ; SDA direction reg bit = x
   move PD6.SDA,#x         ;
ENDM                       ;

SCL_OUT MACRO x      ; SCL output reg bit = x
   move PO6.SCL,#x         ;
ENDM                       ;

SDA_OUT MACRO x      ; SDA output reg bit = x
   move PO6.SDA,#x         ;
ENDM                       ;

SCL_IN_READ MACRO          ; C = SCL input bit
   move C,PI6.SCL          ;
ENDM                       ;

SDA_IN_READ MACRO          ; C = SDA input bit
   move C,PI6.SDA          ;
ENDM                       ;
; CUSTOMIZATION SECTION - END -------------------------------------------------

; MAXCROS ---------------------------------------------------------------------
WAIT MACRO x         ; delay time = (x+1)*50ns
   move LC[1],#x           ;
   djnz LC[1],$            ;
ENDM                       ;

  EXPORT _hsi2cRecv

  .code

; *****************************************************************************
; hsI2cRecv
;
; DESCRIPTION
;
;   Inits the device for High-Speed I2C transfer and reads LC[0] bytes
;   from the device to SRAM starting at address DP[0]. Note that DPC must
;   be initialized such that DP[0] is in byte mode.
;
; PARAMS (not preserved -> compatible with Rowley Asm language interface)
;
;   A[6]  = Number of bytes to read (limited to available SRAM)
;   A[7]  = SRAM address to save bytes read;
;
; REGISTER USAGE
;
;   A[4]  = not preserved (compatible with Rowley Asm language interface)
;   A[5]  = not preserved (compatible with Rowley Asm language interface)
;   DP[0] = not preserved (compatible with Rowley Asm language interface)
;   LC[1] = not preserved (compatible with Rowley Asm language interface)
;   LC[1] = not preserved (compatible with Rowley Asm language interface)
;   AP    = not preserved (compatible with Rowley Asm language interface)
;
; RETURNS
;
;   A[7]  = 0 if error occured, 1 otherwise
;
; STACK USAGE
;
;   0 bytes
;
; *****************************************************************************
_hsi2cRecv:
   move DP[0],A[7]          ; Load number of bytes to read
   move LC[0],A[6]          ; Load addr to save read bytes
 
   call hsi2cInit           ; Init I2C pins

   ; --> I2C START - FULL SPEED
   SDA_DIR 1                ; FS S - drive SDA low
   WAIT 9                   ; FS S - T_HD_STA
   SCL_DIR 1                ; FS S - drive SCL low
   WAIT 19                  ; FS S - T_LOW

   move A[4],#0             ; Init A[0]=0
   move AP  ,#5             ; Acc->A[5] 
   move NUL ,@DP[0]--       ; Init DP[0]=DP[0]-1 because PRE-incr is used later

   ; --> I2C SEND INIT HS ADDR - FULL SPEED
   SCL_DIR 0                ; FS ADDR 7 - pull-up SCL
   WAIT 9                   ; FS ADDR 7 - T_HIGH
   SCL_DIR 1                ; FS ADDR 7 - drive SCL low
   WAIT 23                  ; FS ADDR 7 - T_LOW
   SCL_DIR 0                ; FS ADDR 6 - pull-up SCL
   WAIT 9                   ; FS ADDR 6 - T_HIGH
   SCL_DIR 1                ; FS ADDR 6 - drive SCL low
   WAIT 23                  ; FS ADDR 6 - T_LOW
   SCL_DIR 0                ; FS ADDR 5 - pull-up SCL
   WAIT 9                   ; FS ADDR 5 - T_HIGH
   SCL_DIR 1                ; FS ADDR 5 - drive SCL low
   WAIT 23                  ; FS ADDR 5 - T_LOW
   SCL_DIR 0                ; FS ADDR 4 - pull-up SCL
   WAIT 9                   ; FS ADDR 4 - T_HIGH
   SCL_DIR 1                ; FS ADDR 4 - drive SCL low
   WAIT 9                   ; FS ADDR 4 - T_HD_DAT
   SDA_DIR 0                ; FS ADDR 3 - pull-up SDA
   WAIT 11                  ; FS ADDR 3 - T_SU_DAT
   SCL_DIR 0                ; FS ADDR 3 - pull-up SCL
   WAIT 9                   ; FS ADDR 3 - T_HIGH
   SCL_DIR 1                ; FS ADDR 3 - drive SCL low
   WAIT 23                  ; FS ADDR 3 - T_LOW
   SCL_DIR 0                ; FS ADDR 2 - pull-up SCL
   WAIT 9                   ; FS ADDR 2 - T_HIGH
   SCL_DIR 1                ; FS ADDR 2 - drive SCL low
   WAIT 23                  ; FS ADDR 2 - T_LOW
   SCL_DIR 0                ; FS ADDR 1 - pull-up SCL
   WAIT 9                   ; FS ADDR 1 - T_HIGH
   SCL_DIR 1                ; FS ADDR 1 - drive SCL low
   WAIT 23                  ; FS ADDR 1 - T_LOW
   SCL_DIR 0                ; FS ADDR 0 - pull-up SCL (SDA high)
   WAIT 9                   ; FS ADDR 0 - T_HIGH
   SCL_DIR 1                ; FS ADDR 0 - drive SCL low
   WAIT 23                  ; FS ADDR 0 - T_LOW

   ; I2C SLAVE NACK - FULL SPEED
   SCL_DIR 0                ; FS SLAVE NACK - pull-up SCL
   WAIT 3                   ; FS SLAVE NACK - T_HIGH_DIV2
   SDA_IN_READ              ; FS SLAVE NACK - C = SDA
   jump NC,hsI2cRecvEnd     ; Error
   WAIT 2                   ; FS SLAVE NACK - T_HIGH_DIV2
   SCL_DIR 1                ; FS SLAVE NACK - drive SCL low
   WAIT 23                  ; FS SLAVE NACK - T_LOW

   ; I2C REPEATED START - HIGH SPEED
   SCL_DIR 0                ; HS Sr - pull-up SCL
   WAIT 5                   ;T was 1, HS Sr - T_SU_STA
   SDA_DIR 1                ; HS Sr - drive SDA low
   WAIT 1 
   SCL_DIR 1                ; HS Sr - drive SCL low
   WAIT 1 

   ; I2C ADDR - HIGH SPEED
   SDA_DIR (!I2C_ADDR7)     ; HS ADDR 7 - set SDA  
   nop                      ; HS ADDR 7 - T_SU_DAT
   SCL_DIR 0                ; HS ADDR 7 - pull-up SCL
   WAIT 5                   ;T was nop, HS ADDR 7 - T_HIGH
   SCL_DIR 1                ; HS ADDR 7 - drive SCL low
   WAIT 2                   ; HS ADDR 6 - T_HD_DAT
   SDA_DIR (!I2C_ADDR6)     ; HS ADDR 6 - set SDA
   WAIT 5                   ;T was nop, HS ADDR 6 - T_SU_DAT
   SCL_OUT 1                ; HS ADDR 6 - drive SCL high
   WAIT 1                   ; HS ADDR 6 - T_HIGH
   SCL_OUT 0                ; HS ADDR 6 - drive SCL low
   WAIT 4                   ;T was 2, HS ADDR 5 - T_HD_DAT
   SDA_DIR (!I2C_ADDR5)     ; HS ADDR 5 - set SDA
   nop                      ; HS ADDR 5 - T_SU_DAT
   SCL_OUT 1                ; HS ADDR 5 - drive SCL high
   WAIT 1                   ; HS ADDR 5 - T_HIGH
   SCL_OUT 0                ; HS ADDR 5 - drive SCL low
   WAIT 2                   ; HS ADDR 4 - T_HD_DAT
   SDA_DIR (!I2C_ADDR4)     ; HS ADDR 4 - set SDA
   nop                      ; HS ADDR 4 - T_SU_DAT
   nop                      ;T added extra nop
   nop                      ;T added extra nop
   SCL_OUT 1                ; HS ADDR 4 - drive SCL high
   WAIT 1                   ; HS ADDR 4 - T_HIGH
   SCL_OUT 0                ; HS ADDR 4 - drive SCL low
   WAIT 2                   ; HS ADDR 3 - T_HD_DAT
   SDA_DIR (!I2C_ADDR3)     ; HS ADDR 3 - set SDA
   nop                      ; HS ADDR 3 - T_SU_DAT
   nop                      ;T added extra nop
   nop                      ;T added extra nop
   SCL_OUT 1                ; HS ADDR 3 - drive SCL high
   WAIT 1                   ; HS ADDR 3 - T_HIGH
   SCL_OUT 0                ; HS ADDR 3 - drive SCL low
   WAIT 2                   ; HS ADDR 2 - T_HD_DAT
   SDA_DIR (!I2C_ADDR2)     ; HS ADDR 2 - set SDA
   nop                      ; HS ADDR 2 - T_SU_DAT
   nop                      ;T added extra nop
   nop                      ;T added extra nop
   SCL_OUT 1                ; HS ADDR 2 - drive SCL high
   WAIT 1                   ; HS ADDR 2 - T_HIGH
   SCL_OUT 0                ; HS ADDR 2 - drive SCL low
   WAIT 2                   ; HS ADDR 1 - T_HD_DAT
   SDA_DIR (!I2C_ADDR1)     ; HS ADDR 1 - set SDA
   nop                      ; HS ADDR 1 - T_SU_DAT
   nop                      ;T added extra nop
   nop                      ;T added extra nop
   SCL_OUT 1                ; HS ADDR 1 - drive SCL high
   WAIT 1                   ; HS ADDR 1 - T_HIGH
   SCL_OUT 0                ; HS ADDR 1 - drive SCL low
   WAIT 2                   ; HS ADDR 0 - T_HD_DAT
   SDA_DIR 0                ; HS ADDR 0 - pull-up SDA (R/W=Read)
   nop                      ; HS ADDR 0 - T_SU_DAT
   nop                      ;T added extra nop
   nop                      ;T added extra nop
   SCL_OUT 1                ; HS ADDR 0 - drive SCL high
   WAIT 1                   ; HS ADDR 0 - T_HIGH
   SCL_OUT 0                ; HS ADDR 0 - drive SCL low
   WAIT 7                   ;T was 5, HS ADDR 0 - T_LOW

   ; I2C SLAVE ACK - HIGH SPEED
   SCL_OUT 1                ; HS SLAVE ACK - drive SCL high
   nop                      ; HS SLAVE ACK - T_HIGH
   SDA_IN_READ              ; HS SLAVE ACK - C = SDA
   SCL_OUT 0                ; HS SLAVE ACK - drive SCL low
   jump C,hsI2cRecvEnd      ; HS SLAVE ACK - Error
   WAIT 2                   ; HS CLK STR - T_HD_DAT

hsI2cRecvData:
   ; I2C CLOCK STRETCH - HIGH SPEED
   SCL_DIR 0                ; HS CLK STR - pull-up SCL
   ;   nop                  ; HS CLK STR - safety NOP
hsI2cRecvClkStr:            ; HS CLK STR - 
   SCL_IN_READ              ; HS CLK STR - C = SCL
   jump NC, hsI2cRecvClkStr ; HS CLK STR - If SCL=0 -> wait...

   ; I2C RECV DATA (MSB) - HIGH SPEED
   SDA_IN_READ              ; HS DATA 15 - C = SDA
   move Acc.7,C             ; HS DATA 15 - Acc.15 = C (=SDA)
   SCL_DIR 1                ; HS DATA 15 - drive SCL low
   WAIT 5                   ; HS DATA 15 - T_LOW
   SCL_OUT 1                ; HS DATA 14 - drive SCL high
   SDA_IN_READ              ; HS DATA 14 - C = SDA
   move Acc.6,C             ; HS DATA 14 - Acc.14 = C (=SDA)
   SCL_OUT 0                ; HS DATA 14 - drive SCL low
   WAIT 5                   ; HS DATA 14 - T_LOW
   SCL_OUT 1                ; HS DATA 13 - drive SCL high
   SDA_IN_READ              ; HS DATA 13 - C = SDA
   move Acc.5,C             ; HS DATA 13 - Acc.13 = C (=SDA)
   SCL_OUT 0                ; HS DATA 13 - drive SCL low
   WAIT 5                   ; HS DATA 13 - T_LOW
   SCL_OUT 1                ; HS DATA 12 - drive SCL high
   SDA_IN_READ              ; HS DATA 12 - C = SDA
   move Acc.4,C             ; HS DATA 12 - Acc.12 = C (=SDA)
   SCL_OUT 0                ; HS DATA 12 - drive SCL low
   WAIT 5                   ; HS DATA 12 - T_LOW
   SCL_OUT 1                ; HS DATA 11 - drive SCL high
   SDA_IN_READ              ; HS DATA 11 - C = SDA
   move Acc.3,C             ; HS DATA 11 - Acc.11 = C (=SDA)
   SCL_OUT 0                ; HS DATA 11 - drive SCL low
   WAIT 5                   ; HS DATA 11 - T_LOW
   SCL_OUT 1                ; HS DATA 10 - drive SCL high
   SDA_IN_READ              ; HS DATA 10 - C = SDA
   move Acc.2,C             ; HS DATA 10 - Acc.10 = C (=SDA)
   SCL_OUT 0                ; HS DATA 10 - drive SCL low
   WAIT 5                   ; HS DATA 10 - T_LOW
   SCL_OUT 1                ; HS DATA 09 - drive SCL high
   SDA_IN_READ              ; HS DATA 09 - C = SDA
   move Acc.1,C             ; HS DATA 09 - Acc.9 = C (=SDA)
   SCL_OUT 0                ; HS DATA 09 - drive SCL low
   WAIT 5                   ; HS DATA 09 - T_LOW
   SCL_OUT 1                ; HS DATA 08 - drive SCL high
   SDA_IN_READ              ; HS DATA 08 - C = SDA
   move Acc.0,C             ; HS DATA 08 - Acc.8 = C (=SDA)
   SCL_OUT 0                ; HS DATA 08 - drive SCL low
   WAIT 1                   ; HS DATA 08 - T_LOW

   move @++DP[0],Acc        ; Save byte to SRAM
   djnz LC[0],hsi2cRecvAck  ; Goto Master ACK
   jump hsi2cRecvNack       ; Goto Master NACK

hsi2cRecvAck:
   ; I2C MASTER ACK - HIGH SPEED
   SDA_DIR 1                ; HS MTR ACK - drive SDA low
   SCL_OUT 1                ; HS MTR ACK - drive SCL high
   WAIT 1                   ; HS MTR ACK - T_HIGH
   SCL_OUT 0                ; HS MTR ACK - drive SCL low
   SDA_DIR 0                ; HS MTR ACK - pull-up SDA
   WAIT 2                   ; HS MTR ACK - T_LOW

   jump hsI2cRecvData

hsi2cRecvNack:
   ; I2C MASTER ACK - HIGH SPEED
   SCL_OUT 1                ; HS MTR NACK - drive SCL high
   WAIT 1
   SCL_OUT 0                ; HS MTR NACK - drive SCL low
   WAIT 4                   ; HS MTR NACK - T_LOW

   move A[4],#1             ; No error
hsI2cRecvEnd:

   ; I2C STOP - Full Speed
   SDA_DIR 1                ; FS STOP - drive SDA low
   SCL_DIR 1                ; FS STOP - drive SCL low
   WAIT 9                   ; FS STOP - T_LOW
   SCL_DIR 0                ; FS STOP - pull-up SCL
   WAIT 9                   ; FS STOP - T_SU_STO
   SDA_DIR 0                ; FS STOP - pull-up SDA

   move A[7],A[4]           ; Return error state
   ret                      ; Return from hsi2cRecv ...

; *****************************************************************************
; hsi2cInit - DO NOT CALL THIS ROUTINE DIRECTLY,
;             USE hsi2cRecv OR hsi2cSend
; *****************************************************************************
hsi2cInit:
   SCL_DIR 0                ; Set SCL as input
   SDA_DIR 0                ; Set SDA as input
   SCL_OUT 0                ; Set SCL output low (disable internal pull-up)
   SDA_OUT 0                ; Set SDA output low (disable internal pull-up)
   ret

  end

